package ehu.student.collections;

import java.util.ArrayList;
import java.util.TreeSet;

/**
 * Una clase para crear todas las permutaciones de un conjunto de palabras.
 */
public class Permutador4ArrayListString {

    /**
     * Devuelve una NUEVA lista con las mismas palabras que "lista"
     */
    public ArrayList<String> clonar(ArrayList<String> lista){
        ArrayList<String> r = new ArrayList<String>();
        r.addAll(lista); //aadir todos los elementos de "lista"
        return r;
    }
    
    /**
     * Devuelve todas las listas de palabras que se pueden formar intercalando
     * la palabra "p" en cada una de las posiciones de una "lista" de palabras
     */
    public ArrayList<ArrayList<String>> combinar(ArrayList<String> lista,
                                                 String s)
    {
        ArrayList<ArrayList<String>> listas = new ArrayList<ArrayList<String>>();
        for (int i = 0; i <= lista.size(); i++){
            ArrayList<String> tmp = clonar(lista);
            tmp.add(i, s);   /* INSERTAR "s" en la posicion "i" */
            listas.add(tmp); /* aadir "tmp" a las listas ya calculadas */
        }
        return listas;
    }
    
    /**
     * Devuelve todas las listas de palabras que se pueden formar intercalando
     * la palabra "p" en cada una de las posiciones de cada una de las
     * listas de palabras incluidas en "listas"
     */
    public ArrayList<ArrayList<String>> extender(ArrayList<ArrayList<String>> listas, 
                                                 String p)
    {
        ArrayList<ArrayList<String>> ext = new ArrayList<ArrayList<String>>();                    
        for (ArrayList<String> l: listas){
            ArrayList<ArrayList<String>> rs = combinar(l, p);
            ext.addAll(rs); /* aadir todas las listas de palabras de "rs" */
        }
        return ext;     
    }
    
    /**
     * Devuelve todas las listas de palabras que se pueden formar permutando
     * las palabras de un conjunto.
     */
    public ArrayList<ArrayList<String>> permutar(TreeSet<String> palabras)
    {
        ArrayList<ArrayList<String>> perm = new ArrayList<ArrayList<String>>();
        
        perm.add(new ArrayList<String>());
        /*
         * Las listas de palabras de "perm" se extienden con una palabra;
         * las nuevas listas se extienden con otra palabra,
         * y las nuevas con otra, y asi sucesivamente.
         * Las listas de palabras obtenidas finalmente son las permutaciones
         * del conjunto de palabras.
         */
        for (String s: palabras){
            perm = extender(perm, s);
        }
        return perm;
    }

    /**
     * Devuelve un nuevo String con todas las listas de palabras de "listas",
     * incluyendo un separador de lnea entre cada dos listas.
     */
    public String toString(ArrayList<ArrayList<String>> listas){
        String buff = "";
        for (ArrayList<String> x: listas){
            buff = buff + x + "\n";
        }
        return buff;
    }   
}
